@@ -188,6 +188,7 @@ testthat::test_that("authenticated flips FALSE after expiry without poking react
   # re-evaluation logic. Since invalidateLater() timers don't fire reliably in
   # testServer, we verify the underlying behavior: after time passes, any
   # reactive flush should pick up the expired state.
+  testthat::skip_on_cran() # Timing-sensitive test; may be flaky on slow CRAN machines
   withr::local_options(list(shinyOAuth.skip_browser_token = TRUE))
 
   cli <- make_test_client(use_pkce = TRUE, use_nonce = FALSE)
@@ -237,6 +238,7 @@ testthat::test_that("authenticated flips FALSE after expiry without poking react
 
 testthat::test_that("authenticated flips FALSE after reauth_after_seconds without poking", {
   # Same approach as expiry test - verify the logic works when time passes.
+  testthat::skip_on_cran() # Timing-sensitive test; may be flaky on slow CRAN machines
   withr::local_options(list(shinyOAuth.skip_browser_token = TRUE))
 
   cli <- make_test_client(use_pkce = TRUE, use_nonce = FALSE)
@@ -494,6 +496,9 @@ testthat::test_that("provider error in query sets error and authenticated FALSE"
     ),
     session = sess,
     expr = {
+      # Flush any pending reactive events from module initialization
+      session$flushReact()
+
       values$.process_query("?error=access_denied&error_description=Nope")
       session$flushReact()
       testthat::expect_identical(values$error, "access_denied")
@@ -646,11 +651,13 @@ testthat::test_that("callback_max_query_bytes option is enforced", {
       query_bytes <- nchar(query, type = "bytes")
 
       # Too-small cap rejects the query before parsing continues
-      withr::local_options(list(
-        shinyOAuth.callback_max_query_bytes = query_bytes - 1
-      ))
-      values$.process_query(query)
-      session$flushReact()
+      withr::with_options(
+        list(shinyOAuth.callback_max_query_bytes = query_bytes - 1),
+        {
+          values$.process_query(query)
+          session$flushReact()
+        }
+      )
       testthat::expect_identical(values$error, "invalid_callback_query")
       testthat::expect_match(values$error_description %||% "", "query string")
       testthat::expect_null(values$token)
@@ -659,18 +666,20 @@ testthat::test_that("callback_max_query_bytes option is enforced", {
       values$error <- NULL
       values$error_description <- NULL
       called <- FALSE
-      withr::local_options(list(
-        shinyOAuth.callback_max_query_bytes = query_bytes + 1
-      ))
-      testthat::with_mocked_bindings(
-        swap_code_for_token_set = function(client, code, code_verifier) {
-          called <<- TRUE
-          list(access_token = "t", expires_in = 3600)
-        },
-        .package = "shinyOAuth",
+      withr::with_options(
+        list(shinyOAuth.callback_max_query_bytes = query_bytes + 1),
         {
-          values$.process_query(query)
-          session$flushReact()
+          testthat::with_mocked_bindings(
+            swap_code_for_token_set = function(client, code, code_verifier) {
+              called <<- TRUE
+              list(access_token = "t", expires_in = 3600)
+            },
+            .package = "shinyOAuth",
+            {
+              values$.process_query(query)
+              session$flushReact()
+            }
+          )
         }
       )
       testthat::expect_true(called)
@@ -703,8 +712,27 @@ testthat::test_that("callback code/state clears query even when token exchange f
     ),
     session = sess,
     expr = {
+      # Flush any pending reactive events from module initialization
+      # (e.g., the url_search observer with MockShinySession's default ?mocksearch=1)
+      session$flushReact()
+
+      # Ensure browser token is properly initialized before building auth URL.
+      # If this fails, the option didn't propagate correctly to the module.
+      testthat::expect_true(
+        values$has_browser_token(),
+        info = "Browser token should be available in test mode"
+      )
+
       url <- values$build_auth_url()
+      testthat::expect_true(
+        is.character(url) && nchar(url) > 0 && !is.na(url),
+        info = "build_auth_url() should return a valid URL"
+      )
       enc <- parse_query_param(url, "state")
+      testthat::expect_true(
+        is.character(enc) && nchar(enc) > 0 && !is.na(enc),
+        info = "State parameter should be extractable from URL"
+      )
 
       testthat::with_mocked_bindings(
         swap_code_for_token_set = function(client, code, code_verifier) {
@@ -717,7 +745,17 @@ testthat::test_that("callback code/state clears query even when token exchange f
         }
       )
 
-      testthat::expect_identical(values$error, "token_exchange_error")
+      testthat::expect_identical(
+        values$error,
+        "token_exchange_error",
+        info = paste0(
+          "error=",
+          values$error,
+          "; ",
+          "desc=",
+          values$error_description %||% "(none)"
+        )
+      )
       testthat::expect_true(
         any(seen == "shinyOAuth:clearQueryAndFixTitle"),
         info = "Expected clearQueryAndFixTitle on token-exchange error"
